﻿using System.Collections.Generic;
using System.Linq;
using System.Xml;
using Nehta.VendorLibrary.CdaExtractor.Model;

namespace Nehta.VendorLibrary.CdaExtractor.Extractor
{
    public class ImmunisationsExtractor : ICdaExtractor<List<Immunisation>>
    {
        private readonly IDictionary<string, string> _documentXPaths;

        public ImmunisationsExtractor(IDictionary<string, string> documentXPaths)
        {
            _documentXPaths = documentXPaths;
        }

        public List<Immunisation> Extract(CdaXmlDocument document)
        {
            var nodes = document.SelectNodes(_documentXPaths.Get("Immunisation"));
            if(nodes == null || nodes.Count == 0) return null;

            return
                (from XmlNode node in nodes
                 select new Immunisation
                    {
                        Medicine = document.GetRelativeCode(node, _documentXPaths.Get("ImmunisationItem")),
                        SequenceNumber = document.GetString(node, _documentXPaths.Get("VaccineSequenceNumber")),
                        DateTime = document.GetDateTimeValue(node, _documentXPaths.Get("MedicationActionDate"))
                    }
                ).ToList();
        }
    }
}
